package container;




import datapacker.AbstractDataPacker;

import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 实体类装载器的容器
 *
 * @author 修唯xiuwei
 * @version 3.0
 */
public class DataPackerContainer {


	/**
	 * 各种设备状态对应的service
	 */
	RecurringTaskContainer recurringTaskContainer = RecurringTaskContainer.getInstance();

	/**
	 * packer 的池子
	 */
	private Map<String, AbstractDataPacker> packers = new ConcurrentHashMap<>();

	private Map<String, Callable> tasks = new ConcurrentHashMap<>();

	private DataPackerContainer() {
	}

	/**
	 * 返回容器对象
	 *
	 * @return the instance
	 */
	public static final DataPackerContainer getInstance() {
		return LazyHolder.INSTANCE;
	}

	/**
	 * 添加packer  并启动定时任务
	 *
	 * @param equipmentNo 设备NO
	 * @param packer      组装器
	 * @return {@link  AbstractDataPacker}
	 */
	public synchronized Map<String, AbstractDataPacker> addPacker(String equipmentNo, AbstractDataPacker packer) {
		this.packers.put(equipmentNo, packer);
		tasks.put(equipmentNo, startPackageTask(packer));
		return this.packers;
	}

	/**
	 * 删除组装器
	 * 移除packer
	 *
	 * @param equipmentNo 设备号
	 * @return {@link AbstractDataPacker}
	 */
	public synchronized Map<String, AbstractDataPacker> removePacker(String equipmentNo) {
		this.packers.remove(equipmentNo);
		recurringTaskContainer.removeTask(this.tasks.get(equipmentNo));
		this.tasks.remove(equipmentNo);
		return this.packers;
	}

	/**
	 * 获取packer
	 *
	 * @param equipmentNo 设备No
	 * @return {@link AbstractDataPacker}
	 */
	public synchronized AbstractDataPacker getPacker(String equipmentNo) {
		return this.packers.get(equipmentNo);
	}

	/**
	 * 根据新增的设备 增加定时入库的定时任务
	 *
	 * @param equipmentDataPacker 设备数据组装器
	 * @return {@link Callable}
	 */
	public Callable startPackageTask(AbstractDataPacker equipmentDataPacker) {
	/*	Class clazz = equipmentDataPacker.getRealtimeData().getClass();
		switch (clazz.getSimpleName()) {
			case "AgcRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入AGC实时数据任务", () -> SpringUtils.getBean(AgcRealtimeDataServiceImpl.class).save((AgcRealtimeData) equipmentDataPacker.packageData()));
			case "AvcRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入AVC实时数据任务", () -> SpringUtils.getBean(AvcRealtimeDataServiceImpl.class).save((AvcRealtimeData) equipmentDataPacker.packageData()));
			case "EmsRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入能管平台实时数据任务", () -> SpringUtils.getBean(EmsRealtimeDataServiceImpl.class).save((EmsRealtimeData) equipmentDataPacker.packageData()));
			case "PvInverterRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入逆变器实时数据任务", () -> SpringUtils.getBean(IPvInverterRealtimeDataService.class).save((PvInverterRealtimeData) equipmentDataPacker.packageData()));
			case "WindTurbineRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入风机实时数据任务", () -> SpringUtils.getBean(IWindTurbineRealtimeDataService.class).save((WindTurbineRealtimeData) (equipmentDataPacker.packageData())));
			case "SvgRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入SVG实时数据任务", () -> SpringUtils.getBean(ISvgRealtimeDataService.class).save((SvgRealtimeData) equipmentDataPacker.packageData()));
			case "CollectionLineRealtimeData":
				return recurringTaskContainer.addRecurringTask(equipmentDataPacker.getInterval(), "存入集电线实时数据任务", () -> SpringUtils.getBean(ICollectionLineRealtimeDataService.class).save((CollectionLineRealtimeData) equipmentDataPacker.packageData()));
			default:
				return null;
		}*/
		return null;
	}





	private static class LazyHolder {
		private static final DataPackerContainer INSTANCE = new DataPackerContainer();
	}

}
