package entity.datapoint;

import com.baomidou.mybatisplus.annotation.EnumValue;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import entity.EquipmentAttribute;
import enums.DataType;
import lombok.Getter;
import lombok.Setter;


/**
 * 点表的抽象类
 *
 * @author: xiuwei
 * @version:
 */
@Setter
@Getter
public class BaseDataPoint {

    @TableField(exist = false)
    protected DataPointType dataPointType;

    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * 设备编号
     */
    private Integer equipmentNo;

    /**
     * 协议数据类型
     */
    @EnumValue
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private DataType dataType;

    /**
     * 设备属性
     */
     @TableField(exist = false)
    private EquipmentAttribute equipmentAttribute;
    /**
     * 设备属性的ID冗余字段
     */
    private Integer equipmentAttributeId;

    public void setEquipmentAttribute(EquipmentAttribute equipmentAttribute) {
        this.equipmentAttribute = equipmentAttribute;
        if (this.equipmentAttributeId == null || !this.equipmentAttributeId.equals(equipmentAttribute.getId())) {
            this.equipmentAttributeId = equipmentAttribute.getId();
        }
    }

    public void setEquipmentAttributeId(Integer equipmentAttributeId) {
        this.equipmentAttributeId = equipmentAttributeId;
        if (this.equipmentAttribute == null || !equipmentAttributeId.equals(this.equipmentAttribute.getId())) {
            this.equipmentAttribute = new EquipmentAttribute();
            this.equipmentAttribute.setId(this.equipmentAttributeId);
        }
    }

    public String getSId() {
        if (this instanceof ConfigDataPoint) {
            return "C" + getId();
        } else if (this instanceof GatherDataPoint) {
            return "G" + getId();
        } else if (this instanceof CalculateDataPoint) {
            return "J" + getId();
        }
        return getId() + "";
    }

    public enum DataPointType {
        CALCULATE, CONFIG, GATHER
    }

}
