package tunnelworker.masters.iml;


import enums.TunnelStatus;
import exception.DataExchangeException;
import mastertunnelinfo.MasterModbusRtuTunnelInfo;
import tunnelworker.BaseProtocolTunnel;
import tunnelworker.workassist.SingleThreadPoolExecutorUtil;
import wei.yigulu.modbus.netty.ModbusRtuMasterBuilder;
import wei.yigulu.purejavacomm.PureJavaCommChannelConfig;

/**
 * modbus RTU 采集通道
 *
 * @author: xiuwei
 * @version: 3.0
 */
public class ModbusRtuMaster extends BaseModbusMaster<MasterModbusRtuTunnelInfo, ModbusRtuMasterBuilder> {


    /**
     * 构造方法
     *
     * @param gatherModbusRtuTunnelInfo 通道信息
     */
    public ModbusRtuMaster(final MasterModbusRtuTunnelInfo gatherModbusRtuTunnelInfo) {
        super(gatherModbusRtuTunnelInfo);
    }

    @Override
    protected ModbusRtuMaster buildTunnel() throws DataExchangeException {
        setSlaveId(tunnelInfo.getSlaveId());
        protocolBuilder = new ModbusRtuMasterBuilder(this.tunnelInfo.getSerialName());
        protocolBuilder.setBaudRate(tunnelInfo.getBaudRate());
        protocolBuilder.setDataBits(PureJavaCommChannelConfig.Databits.valueOf(tunnelInfo.getDataBits()));
        protocolBuilder.setStopBits(PureJavaCommChannelConfig.Stopbits.valueOf(tunnelInfo.getStopBits()));
        protocolBuilder.setParity(PureJavaCommChannelConfig.Paritybit.valueOf(tunnelInfo.getParity()));
        protocolBuilder.setReadTimeOut(500);
        protocolBuilder.setLog(this.log);
        setTunnelStatus(TunnelStatus.BUILT);
        protocolTunnelContainer.addTunnel(this);
        log.info("成功创建ModbusRTUMaster通道对象：{}", this.tunnelInfo.getTunnelName());
        return this;
    }


    @Override
    public ModbusRtuMaster startTunnel() throws DataExchangeException {
        log.info("modbus RTU master {} 通道开始连接", getTunnelInfo().getTunnelName());
        protocolTunnelContainer.addUpdateDateTask(this);
        try {
            SingleThreadPoolExecutorUtil.executeBySingleThreadExecutor(() -> {
                try {
                    this.protocolBuilder.create();
                    log.error("Modbus Rtu Master  创建通道失败");
                    setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
                } catch (Exception e) {
                    log.error("Modbus Rtu Master  创建通道失败", e);
                    setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
                }
            });
        } catch (Exception e) {
            log.error("Modbus Rtu Master 创建通道失败", e);
            setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
            throw new DataExchangeException(10007, "Modbus Rtu Master创建通道失败");
        }
        if (!TunnelStatus.LISTENSERIALFAIL.equals(getTunnelStatus())) {
            setTunnelStatus(TunnelStatus.LISTENSERIALSUCCESS);
        }
        return this;
    }

    @Override
    public void getData() throws DataExchangeException {
        try {
            super.getData();
        } catch (DataExchangeException e) {
            this.setTunnelStatus(TunnelStatus.LISTENPORTSUCCESSANDCONNANDCOMMERROR);
            throw e;
        }
        this.setTunnelStatus(TunnelStatus.LISTENSERIALSUCCESS);
    }

    @Override
    public BaseProtocolTunnel tunnelStop() throws DataExchangeException {
        if (this.protocolBuilder != null) {
            this.protocolBuilder.stop();
        }
        log.info("关闭 Modbus Rtu Master 通道 {}", this.tunnelInfo.getTunnelName());
        return super.tunnelStop();
    }


}


